// **********************************************************************
//
// Copyright (c) 2003
// ZeroC, Inc.
// Billerica, MA, USA
//
// All Rights Reserved.
//
// Ice is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation.
//
// **********************************************************************

#include <Ice/Ice.h>
#include <HelloSessionI.h>
#include <IceUtil/UUID.h>

using namespace std;

HelloSessionManagerI::HelloSessionManagerI(const Ice::ObjectAdapterPtr& adapter) :
    _adapter(adapter)
{
}

Glacier::SessionPrx
HelloSessionManagerI::create(const string& userId, const Ice::Current&)
{
    Glacier::SessionPtr session = new HelloSessionI(userId, this, _adapter);
    Ice::Identity ident;
    ident.category = userId;
    ident.name = "session";

    _adapter->add(session, ident);
    return Glacier::SessionPrx::uncheckedCast(_adapter->createProxy(ident));
}

void
HelloSessionManagerI::remove(const Ice::Identity& ident)
{
    _adapter->remove(ident);
}

HelloSessionI::HelloSessionI(const string& userId,
			     const HelloSessionManagerIPtr& manager,
			     const ::Ice::ObjectAdapterPtr& adapter ) :
   _userId(userId),
   _manager(manager),
   _adapter(adapter)
{
   Ice::Identity contractIdent;
   contractIdent.category = userId;
   contractIdent.name = IceUtil::generateUUID();

   FooContractPtr contract = new FooContractI();
   _contract = FooContractPrx::uncheckedCast( _adapter->createProxy( contractIdent ) );
   _adapter->add( contract, contractIdent );
}

void
HelloSessionI::destroy(const Ice::Current& current)
{
    _adapter->remove( _contract->ice_getIdentity() );
    _manager->remove(current.id);
}

void
HelloSessionI::hello(const Ice::Current&)
{
    cout << "Hello " << _userId << endl;
}

FooContractPrx
HelloSessionI::getContract( const Ice::Current& ) const
{
   return _contract;
}
